#####################################################################################################################
# RTRIM-shell. Version RTRIM-shell_v1.3
# Tool to run rtrim (= TRIM in R) for multiple species and subsets of sites.
# Marnix de Zeeuw rtrim@cbs.nl Statistics Netherlands 2019. 
#####################################################################################################################

#####################################################################################################################
# EXPLANATION OF THE FILES USED AS INPUT. 
#####################################################################################################################

# The code in this shell reads files each of which contain counts for a particular combination of species and stratum (stratum is e.g. a region).
# These counts are analysed using rtrim. 
# The most simple format of the file containing the counts is:
#
# site year count
#    3 1994    NA
#    3 1995     1
#    3 1996     2
#
# Use "NA" for missing values instead of -1 as in the old trim version (in delphi).
#
# If weights are available, the format is:
#
# site year count weights
#    3 1994    NA       1
#    3 1995     1       1
#    3 1996     2       1
#
# When no weights are used, then don't include that column.
#
# File name: for proper data processing the name of the file containing the counts must be a combination of letters (e.g. REP), 
# followed by an underscore and a number that represents the species number,  
# followed by an underscore and a number that represents the stratum type (1=standard; 2=strata combined, processing the latter however requires a different tool),  
# followed by an underscore and a number that represents the stratum number, 
# followed by "_counts.csv".
# e.g. REP_411_1_1_counts.csv
# In the example, REP stands for reptiles, 411 is the species code of sandlizard, 1 refers to stratumtype and 1 to statumnumber (e.g. coastal dune region)
#
# In addition to a file with count data, rtrim requires arguments to inform how the analysis should be done. 
# For each file with count data, an associated argument file is needed, with a similar name, except for the last part: "_arg_input_stratum.csv" instead of "_counts.csv" .
# Thus, datafile REP_411_1_1_counts.csv needs to be accompanied by arguments file REP_411_1_1_arg_input_stratum.csv.

# Format of this file is as follows:
#
# File        Base_year_first_year Base_year_last_year Changepoints Serial_correlation Overdispersion Presence_weights Presence_monthfactors Year_from Save_fitted_values  
# REP_411_1_1 1990                 1990                all          TRUE               TRUE           TRUE             FALSE	             2007      TRUE               
#
# File: short name of file to be analysed, i.e., datafilename without _"counts.csv"

# Rather than chosing one particular base year (as in the old TRIM), it is also possible to use a number of years as base period.
# This is not recommended for the PECBMS data, use first year as Base_year_first_year and Base_year_last_year, please. # 28-02-2022 Eva Silarova
# Base_year_first_year: Calendar year used as base year for indices. 
# Base_year_last_year: Calendar year used as base year for indices.                                                                      
# If Base_year_first_year equals Base_year_last_year a single year is used as base year
# If Base_year_first_year < Base_year_last_year, a period is used as base time
# In the latter case, Base_year_first_year is the first year of the period.

# Changepoints: Changepoints used to calculate the indices. Usually set to "all".
# Serial correlation: Set to "TRUE" when serial correlation needs to be taken into account. Otherwise set to "FALSE".
# Overdispersion: Set to "TRUE" when overdisperion needs to be taken into account. Otherwise set to "FALSE".
# Presence_weights: TRUE when weights are available, FALSE when not.
# Presence_monthfactors: TRUE when available, FALSE when not.
# Year_from: first calendar year of the subperiod over which also a slope needs to be computed.
# Last year of the subperiod equals the last year of the entire period.
# Save_fitted_values: Set to "TRUE" when the fitted values need to be saved. Otherwise set to "FALSE".

# rtrim starts with trying the most elaborate model.
# When the analysis of that model fails, a less elaborate model is tried.
# Output for each species is stored in a RData-file, in the working directory with the count files.
# That output needs to be processed with a separate script ("processingOutputSpeciesStratumCombinations.r").

# rtrim enables to run models with covariates, but this rtrim shell does not support that option.  
# For more information we refer to the trim-vignettes in R.

#Changes to be done in the script:
# 1. Line 86: Adjust the path to the file called functions_Shell_Rtrim.R according to your computer (try different slashes, if double backslash does not work)
# 2. Line 93: Adjust the path to your working folder according to your computer (try different slashes, if double backslash does not work)

#####################################################################################################################
# PREPARATION.
#####################################################################################################################
                                                                                                    # 1-06-2020 Eva Silarova unified upper and lower cases in variable names - v1.1
rm(list=ls())

# Link to script with functions   !! user dependent !!
source("D://webinar//02_RTRIM_shell_v1.4//functions_Shell_Rtrim.R")

# The required library
library(rtrim)

# Select the directory which contains the RTRIM output files
# This directory also contains the files with arguments used in the rtrim function.
  folder <- "D://webinar//02_RTRIM_shell_v1.4"  # !! user dependent!! 
# folder <- choose.dir(default ="D:/WORK/RTRIM-shell/Examples_for_coordinators_Input-files/00_RTRIM-shell_inputs&scripts_corrected_AG", caption = "Select directory which contains data which has been analysed")
setwd(folder)

# Select all files in the directory which contain arguments.
# The files can be recognized by the pattern "arg_input_arguments".
listSpeciesStratumCombinations <- dir(folder, pattern = "arg_input_stratum.csv")

# The number of files in the directory.                                                
numberSpeciesStratumCombinations <- length (listSpeciesStratumCombinations)

# Make a table for an overview. 
# The tabel lists which analyses were successful or not for all runs 

overview <- makeOverview(listSpeciesStratumCombinations)
# The code of this function can be found in functions_Shell_Rtrim.R.
# The source command (see above) makes it possible to use this function in this code.

#####################################################################################################################
# READING ARGUMENT AND COUNT FILES. 
#####################################################################################################################

for (j in 1:numberSpeciesStratumCombinations) {

# j<- 1

# The file with arguments contains the information to analyse the counts for a particular combination of species and stratum. 
# The arguments are used when calling the function 'rtrim'
  arguments <- read.csv2(listSpeciesStratumCombinations[j], header = TRUE, stringsAsFactors = FALSE)

# The file with counts contains the counts for a particular combination of species and stratum.
# Weights may also be present in this file.
  counts <- read.csv2(paste(arguments$File, "_counts.csv", sep = ""), stringsAsFactors = FALSE, header = TRUE)
  
  overview$first_year[j] <- min(counts$year, na.rm = TRUE)
  overview$last_year[j]  <- max(counts$year, na.rm = TRUE)
# First and last year for each combination of stratum and species is stored for later use.
  
#####################################################################################################################
# RUNNING RTRIM.
#####################################################################################################################

# Start with the most elaborate model and switch automatically to a more simple model when needed.

  result <- tryCatch(
    {
      # Attempt 1
      if (arguments$Presence_weights == TRUE & arguments$Presence_monthfactors == TRUE) {
        trim(count ~ site + (year + month), data = counts, weights = "weights", model = 2, changepoints = arguments$Changepoints, serialcor = FALSE, overdisp = arguments$Overdispersion, max_iter = 200, conv_crit = 1e-5)
    } else {
        if (arguments$Presence_weights == TRUE & arguments$Presence_monthfactors == FALSE) {
          trim(count ~ site + year, data = counts, weights = "weights", model = 2, changepoints = arguments$Changepoints, serialcor = arguments$Serial_correlation, overdisp = arguments$Overdispersion, max_iter = 200, conv_crit = 1e-5)
        } else{
            if(arguments$Presence_weights == FALSE & arguments$Presence_monthfactors == TRUE) {
              trim(count ~ site + (year + month), data = counts,            model = 2, changepoints = arguments$Changepoints, serialcor = FALSE, overdisp = arguments$Overdispersion, max_iter = 200, conv_crit = 1e-5)    
          } else{
              trim(count ~ site + year, data = counts,                      model = 2, changepoints = arguments$Changepoints, serialcor = arguments$Serial_correlation, overdisp = arguments$Overdispersion, max_iter = 200, conv_crit = 1e-5)    
          }
        }
      }
    }
    , error = warning)     
  
  if (class(result) == "trim") {

    save(x = result,  file = paste(arguments$File, ".RData", sep = ""))
    overview$attempt_1[overview$ss_combinations == arguments$File] <- "success"
    overview$success[overview$ss_combinations == arguments$File] <- "yes"
  
  } else {

# First attempt failed, try a less elaborate model by setting serial correlation off. 
# Also, when month factors are available, no changepoints are estimated in the next model.
    
      overview$attempt_1[overview$ss_combinations == arguments$File] <- "error"
      overview$error_1[overview$ss_combinations == arguments$File] <- result
            
      result <- tryCatch(
        {
        # attempt 2 
        if (arguments$Presence_weights == TRUE & arguments$Presence_monthfactors == TRUE) {
          trim(count ~ site + (year + month), data = counts, weights = "weights", model = 2,                               serialcor = FALSE, overdisp = arguments$Overdispersion, max_iter = 200, conv_crit = 1e-5)
      } else {
          if (arguments$Presence_weights == TRUE & arguments$Presence_monthfactors == FALSE) {
            trim(count ~ site + year, data = counts, weights = "weights", model = 2, changepoints = arguments$Changepoints, serialcor = FALSE, overdisp = arguments$Overdispersion, max_iter = 200, conv_crit = 1e-5)
        } else{
            if(arguments$Presence_weights == FALSE & arguments$Presence_monthfactors == TRUE) {
              trim(count ~ site + (year + month), data = counts,                      model = 2,                               serialcor = FALSE, overdisp = arguments$Overdispersion, max_iter = 200, conv_crit = 1e-5)    
          } else{
              trim(count ~ site + year, data = counts,                      model = 2, changepoints = arguments$Changepoints, serialcor = FALSE, overdisp = arguments$Overdispersion, max_iter = 200, conv_crit = 1e-5)    
            }
          }
        }
      }
      , error = warning)
        
      if (class(result) == "trim") {
      
        save(x = result,  file = paste(arguments$File, ".RData", sep = ""))
        overview$attempt_2[overview$ss_combinations == arguments$File] <- "success"
        overview$success[overview$ss_combinations == arguments$File] <- "yes"
    
      } else {
    
# Second attempt also failed. Now try an even more simple model: no month factors, no changepoints, but serial correlation switched on. 
# Note that no further options are available to include month factors in the model. 
          
          overview$attempt_2[overview$ss_combinations == arguments$File] <- "error"
          overview$error_2[overview$ss_combinations == arguments$File] <- result
        
          if (arguments$Presence_monthfactors == TRUE) {
            
            cat("Analysis failed for this combination of species and stratum:", arguments$File, "\n")
            
          }
          
          result <- tryCatch( 
            {
              # attempt 3
             
                if (arguments$Presence_weights == TRUE & arguments$Presence_monthfactors == FALSE) {
                  
                  trim(count ~ site + year, data = counts, weights = "weights", model = 2, serialcor = TRUE, overdisp = arguments$Overdispersion, max_iter = 200, conv_crit = 1e-5)
              
              } else{
                  
                  if (arguments$Presence_weights == FALSE & arguments$Presence_monthfactors == FALSE) {
                    trim(count ~ site + year, data = counts,                      model = 2, serialcor = TRUE, overdisp = arguments$Overdispersion, max_iter = 200, conv_crit = 1e-5)    
                  }
              }
            }
            , error = warning)
    
          if (class(result) == "trim") {
      
            save(x = result,  file = paste(arguments$File, ".RData", sep = ""))
            overview$attempt_3[overview$ss_combinations == arguments$File] <- "success"
            overview$success[overview$ss_combinations == arguments$File] <- "yes"
      
          } else {
              if (arguments$Presence_monthfactors == FALSE) {
                overview$attempt_3[overview$ss_combinations == arguments$File] <- "error"
                overview$error_3[overview$ss_combinations == arguments$File] <- result
              }

# Third attempt also failed. Now try the most simple model: no changepoints at all and no serial correlation. 

              result <- tryCatch(
                {
                  # Final attempt
                  
                    if (arguments$Presence_weights == TRUE & arguments$Presence_monthfactors == FALSE) {
                      trim(count ~ site + year, data = counts, weights = "weights", model = 2, serialcor = FALSE, overdisp = arguments$Overdispersion, max_iter = 200, conv_crit = 1e-5)
                  } else{
                      if (arguments$Presence_weights == FALSE & arguments$Presence_monthfactors == FALSE) {
                        trim(count ~ site + year, data = counts,                      model = 2, serialcor = FALSE, overdisp = arguments$Overdispersion, max_iter = 200, conv_crit = 1e-5)    
                      } 
                    }
                  
                }
                
                , error = warning)
            
              if (class(result) == "trim") {
              
               save(x = result,  file = paste(arguments$File, ".RData", sep = ""))
               overview$attempt_4[overview$ss_combinations == arguments$File] <- "success"
               overview$success[overview$ss_combinations == arguments$File] <- "yes"
              
              } else {
                if (arguments$Presence_monthfactors == FALSE) {
                                
# When this analysis also fails, send a error message to screen.
                  
                  overview$attempt_4[overview$ss_combinations == arguments$File] <- "error"
                  overview$error_4[overview$ss_combinations == arguments$File] <- result
                
                  cat("Analysis failed for this combination of species and stratum:", arguments$File, "\n")
          }        
        }
      }  
    } 
  }  
}

#####################################################################################################################
# WRITING OVERVIEW OF RTRIM SUCCESSES and FAILURES.  
#####################################################################################################################
overview <- overview[order(overview$species_number, overview$stratum_number), ]
write.csv2(overview, "overview.csv", row.names = FALSE)
                                              

                                                                           
